/* FLBINIT.C Initialization procedures for Fuzzy Logic Broom Balancing */

#ifndef FLBINIT_H
#define FLBINIT_H

#include <alloc.h>
#include "flbrm.h"

void init_flbrm_rules (fuzzy_system_rec *fl) {
   const int
      no_of_x_rules = 9,
      no_of_theta_rules = 9;
   int i;
   for (i = 0;i < no_of_theta_rules;i++) {
       fl->rules[i].inp_index[0] = in_theta;
       fl->rules[i].inp_index[1] = in_theta_dot;
       }
     /* Regions for theta and theta_dot: */
      fl->rules[0].inp_fuzzy_set[0] = in_neg;
      fl->rules[0].inp_fuzzy_set[1] = in_neg;
      fl->rules[0].out_fuzzy_set = out_nl;
      fl->rules[1].inp_fuzzy_set[0] = in_neg;
      fl->rules[1].inp_fuzzy_set[1] = in_ze;
      fl->rules[1].out_fuzzy_set = out_nm;
      fl->rules[2].inp_fuzzy_set[0] = in_neg;
      fl->rules[2].inp_fuzzy_set[1] = in_pos;
      fl->rules[2].out_fuzzy_set = out_ze;
      fl->rules[3].inp_fuzzy_set[0] = in_ze;
      fl->rules[3].inp_fuzzy_set[1] = in_neg;
      fl->rules[3].out_fuzzy_set = out_nm;
      fl->rules[4].inp_fuzzy_set[0] = in_ze;
      fl->rules[4].inp_fuzzy_set[1] = in_ze;
      fl->rules[4].out_fuzzy_set = out_ze;
      fl->rules[5].inp_fuzzy_set[0] = in_ze;
      fl->rules[5].inp_fuzzy_set[1] = in_pos;
      fl->rules[5].out_fuzzy_set = out_pm;
      fl->rules[6].inp_fuzzy_set[0] = in_pos;
      fl->rules[6].inp_fuzzy_set[1] = in_neg;
      fl->rules[6].out_fuzzy_set = out_ze;
      fl->rules[7].inp_fuzzy_set[0] = in_pos;
      fl->rules[7].inp_fuzzy_set[1] = in_ze;
      fl->rules[7].out_fuzzy_set = out_pm;
      fl->rules[8].inp_fuzzy_set[0] = in_pos;
      fl->rules[8].inp_fuzzy_set[1] = in_pos;
      fl->rules[8].out_fuzzy_set = out_pl;
      for (i = 0;i < no_of_x_rules;i++) {
	  fl->rules[i + no_of_theta_rules].inp_index[0] = in_x;
	  fl->rules[i + no_of_theta_rules].inp_index[1] = in_x_dot;
	  }
     /* Regions for x and x_dot: */
      fl->rules[9].inp_fuzzy_set[0] = in_neg;
      fl->rules[9].inp_fuzzy_set[1] = in_neg;
      fl->rules[9].out_fuzzy_set = out_pl;
      fl->rules[10].inp_fuzzy_set[0] = in_neg;
      fl->rules[10].inp_fuzzy_set[1] = in_ze;
      fl->rules[10].out_fuzzy_set = out_ze;
      fl->rules[11].inp_fuzzy_set[0] = in_neg;
      fl->rules[11].inp_fuzzy_set[1] = in_pos;
      fl->rules[11].out_fuzzy_set = out_ze;
      fl->rules[12].inp_fuzzy_set[0] = in_ze;
      fl->rules[12].inp_fuzzy_set[1] = in_neg;
      fl->rules[12].out_fuzzy_set = out_ze;
      fl->rules[13].inp_fuzzy_set[0] = in_ze;
      fl->rules[13].inp_fuzzy_set[1] = in_ze;
      fl->rules[13].out_fuzzy_set = out_ze;
      fl->rules[14].inp_fuzzy_set[0] = in_ze;
      fl->rules[14].inp_fuzzy_set[1] = in_pos;
      fl->rules[14].out_fuzzy_set = out_ze;
      fl->rules[15].inp_fuzzy_set[0] = in_pos;
      fl->rules[15].inp_fuzzy_set[1] = in_neg;
      fl->rules[15].out_fuzzy_set = out_ze;
      fl->rules[16].inp_fuzzy_set[0] = in_pos;
      fl->rules[16].inp_fuzzy_set[1] = in_ze;
      fl->rules[16].out_fuzzy_set = out_ze;
      fl->rules[17].inp_fuzzy_set[0] = in_pos;
      fl->rules[17].inp_fuzzy_set[1] = in_pos;
      fl->rules[17].out_fuzzy_set = out_nl;
      return;
      }

void init_flbrm_mem_fns (fuzzy_system_rec *fl) {
   /* The X membership functions */
   fl->inp_mem_fns[in_x][in_neg] = init_trapz (-1.5,-0.5,0,0,left);
   fl->inp_mem_fns[in_x][in_ze] = init_trapz (-1.5,-0.5,0.5,1.5,regular);
   fl->inp_mem_fns[in_x][in_pos] = init_trapz (0.5,1.5,0,0,right);
   /* The X dot membership functions */
   fl->inp_mem_fns[in_x_dot][in_neg] = init_trapz (-3.0,0,0,0,left);
   fl->inp_mem_fns[in_x_dot][in_ze] = init_trapz (-1.5,-0.5,0.5,1.5,regular);
   fl->inp_mem_fns[in_x_dot][in_pos] = init_trapz (0,3.0,0,0,right);
   /* The theta membership functions */
   fl->inp_mem_fns[in_theta][in_neg] = init_trapz (-0.1,0,0,0,left);
   fl->inp_mem_fns[in_theta][in_ze] = init_trapz (-0.1,-0.025,0.025,0.1,regular);
   fl->inp_mem_fns[in_theta][in_pos] =init_trapz (0,0.1,0,0,right);
   /* The theta dot membership functions */
   fl->inp_mem_fns[in_theta_dot][in_neg] = init_trapz (-0.1,0,0,0,left);
   fl->inp_mem_fns[in_theta_dot][in_ze] = init_trapz (-0.15,-0.025,0.025,0.15,regular);
   fl->inp_mem_fns[in_theta_dot][in_pos] = init_trapz (0,0.1,0,0,right);
   return;
   }

void init_flbrm_fuzzy_system (fuzzy_system_rec *fl) {
   fl->no_of_inputs = 2;  /* Inputs are handled 2 at a time only */
   fl->no_of_rules = 18;
   fl->no_of_inp_regions = 3;
   fl->no_of_outputs = 5;
   fl->output_values [out_nl] = -10.0;
   fl->output_values [out_nm] = -5.0;
   fl->output_values [out_ze] = 0.0;
   fl->output_values [out_pm] = 5.0;
   fl->output_values [out_pl] = 10.0;
   fl->rules = (rule *) malloc ((size_t)(fl->no_of_rules*sizeof(rule)));
   init_flbrm_rules(fl);
   init_flbrm_mem_fns(fl);
   return;
   }

#endif