/*  File FLGRUN.C Run fuzzy logic model in forward mode  */

#ifndef FLGRUN_C
#define FLGRUN_C

#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <alloc.h>

#include "ut.h"
#include "trn.h"
#include "flga.h"

void run_fuzzy_model (tflg_setup_rec fl) {
   fuzzy_system_rec the_fuzzy_system;
   float fl_out = 0,*pfl_out,denorm_fl_out = 0,*pdenorm_fl_out,
	 norm_ans = 0,*pnorm_ans;
   const int one_fl_output = 1;
   training_rec tr_rec = {0,0,1.0,1.0,NULL,NULL,NULL,0,
	  NULL,NULL,NULL,NULL};
   int i;
   FILE *outfile;
   /*  We need to treat output values as arrays (of size 1) that start    */
   /*  at index 1 (so we can interface with the normalization procedures). */
   pfl_out = &fl_out -  1;
   pdenorm_fl_out = &denorm_fl_out - 1;
   pnorm_ans = &norm_ans - 1;
   the_fuzzy_system.allocated = FALSE;
   clrscr ();
   textcolor (YELLOW);
   gotoxy (1,5);
     if (fl.fuzzy_system_file_name[0] == '\0') {
	cprintf ("\r\nFuzzy System File Name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     if (fl.training_file_name[0] == '\0') {
	cprintf ("\r\nTraining File Name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     if (fl.out_data_file_name[0] == '\0') {
	cprintf ("\r\nOutput Data File Name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
   cprintf ("Run Fuzzy Model.");
   gotoxy (1,7);
   cprintf ("Fuzzy System File Name: %s",fl.fuzzy_system_file_name);
   gotoxy (1,9);
   cprintf ("Training File Name: %s",fl.training_file_name);
   gotoxy (1,11);
   cprintf ("Output Data File Name: %s",fl.out_data_file_name);
   gotoxy (1,13);
   cprintf ("Press key to continue (esc to cancel): ");
   if (getch() == ESC_KEY) goto exit_proc;
   clrscr();
   init_flga_fuzzy_system (fl,&the_fuzzy_system);
   if (!process_training_file (fl.training_file_name,
       fl.no_of_fl_inputs,one_fl_output,&tr_rec)) goto exit_proc;
   clrscr();
   gotoxy (1,5);
   cprintf ("Initializing system...");
   normalize_training_set (fl.no_of_fl_inputs,one_fl_output,
      fl.norm_in_range,fl.norm_in_min,fl.norm_out_range,fl.norm_out_min,
      tr_rec);
   if (!read_fuzzy_system (fl.fuzzy_system_file_name,
      fl.no_of_fl_inputs,fl.no_of_fl_inp_regions,
      the_fuzzy_system.no_of_rules,fl.no_of_fl_output_values,
      &the_fuzzy_system)) goto exit_proc;
   if (!open_output_text_file (&outfile,fl.out_data_file_name)) goto exit_proc;
   clrscr();
   for (i = 1;i<=tr_rec.no_of_training_items;i++) {
       fl_out = fuzzy_system (tr_rec.training_array[i],the_fuzzy_system);
       denormalize_output (pfl_out,pdenorm_fl_out,one_fl_output,
	   fl.norm_out_range,fl.norm_out_min,tr_rec);
       normalize_data (tr_rec.correct_answers[i],pnorm_ans,one_fl_output,
	   tr_rec.max_out,tr_rec.min_out,fl.norm_out_range,fl.norm_out_min);
       cprintf ("\r\n%2d: Fz: %8.5f,  Act: %8.5f, Err: %8.5f, NErr: %8.5f",
	     i,denorm_fl_out,tr_rec.correct_answers[i][1],
	     (denorm_fl_out - tr_rec.correct_answers[i][1]),
	     fabs(fl_out - norm_ans));
       fprintf(outfile,"%2d  %8.5f  %8.5f  %8.5f  %8.5f\n",
	     i,denorm_fl_out,tr_rec.correct_answers[i][1],
	     (denorm_fl_out - tr_rec.correct_answers[i][1]),
	     fabs(fl_out - norm_ans));
       }  /*  end i  */
       fclose (outfile);
       cprintf ("\r\nDone.  Data saved to file: %s",fl.out_data_file_name);
       cprintf ("\r\n\nHit key to continue: ");
       getch();
exit_proc:
       free_fuzzy_rules (&the_fuzzy_system);
       free_training_pointers(&tr_rec);
       textmode (LASTMODE);
       return;
       }  /*  end proc  */

#endif

